@@ -15,10 +15,13 @@ import urllib.request
 import uuid
 from typing import IO, Any, Callable, Iterator, Optional, Tuple, TypeVar, Union, cast
 
-import react_ipywidgets as react
+import reacton
 
+import solara
 from solara.datatypes import FileContentResult, Result, ResultState
 
+SOLARA_ALLOW_OTHER_TRACER = os.environ.get("SOLARA_ALLOW_OTHER_TRACER", False) in (True, "True", "true", "1")
+
 logger = logging.getLogger("react-ipywidgets.extra.hooks")
 chunk_size_default = 1024**2
 
@@ -32,6 +35,7 @@ __all__ = [
     "use_uuid4",
     "use_unique_key",
     "use_state_or_update",
+    "use_previous",
 ]
 T = TypeVar("T")
 U = TypeVar("U")
@@ -39,6 +43,18 @@ U = TypeVar("U")
 MaybeResult = Union[T, Result[T]]
 
 
+# not available in python 3.6
+class nullcontext(contextlib.AbstractContextManager):
+    def __init__(self, enter_result=None):
+        self.enter_result = enter_result
+
+    def __enter__(self):
+        return self.enter_result
+
+    def __exit__(self, *excinfo):
+        pass
+
+
 # inherit from BaseException so less change of being caught
 # in an except
 class CancelledError(BaseException):
@@ -46,7 +62,7 @@ class CancelledError(BaseException):
 
 
 def use_retry(*actions: Callable[[], Any]):
-    counter, set_counter = react.use_state(0)
+    counter, set_counter = solara.use_state(0)
 
     def retry():
         for action in actions:
@@ -72,14 +88,14 @@ def use_thread(
     def make_lock():
         return threading.Lock()
 
-    lock: threading.Lock = react.use_memo(make_lock)()
+    lock: threading.Lock = solara.use_memo(make_lock, [])
     updater = use_force_update()
-    result_state, set_result_state = react.use_state(ResultState.INITIAL)
-    error = react.use_ref(cast(Optional[Exception], None))
-    result = react.use_ref(cast(Optional[T], None))
-    running_thread = react.use_ref(cast(Optional[threading.Thread], None))
+    result_state, set_result_state = solara.use_state(ResultState.INITIAL)
+    error = solara.use_ref(cast(Optional[Exception], None))
+    result = solara.use_ref(cast(Optional[T], None))
+    running_thread = solara.use_ref(cast(Optional[threading.Thread], None))
     counter, retry = use_retry()
-    cancel: threading.Event = react.use_memo(make_event)(dependencies + [counter])
+    cancel: threading.Event = solara.use_memo(make_event, [*dependencies, counter])
 
     @contextlib.contextmanager
     def cancel_guard():
@@ -90,11 +106,13 @@ def use_thread(
         def tracefunc(frame, event, arg):
             # this gets called at least for every line executed
             if cancel.is_set():
-                rc = react.core._get_render_context(required=False)
+                rc = reacton.core._get_render_context(required=False)
                 # we do not want to cancel the rendering cycle
                 if rc is None or not rc._is_rendering:
                     # this will bubble up
                     raise CancelledError()
+            if prev and SOLARA_ALLOW_OTHER_TRACER:
+                prev(frame, event, arg)
             # keep tracing:
             return tracefunc
 
@@ -113,7 +131,6 @@ def use_thread(
                 sys.settrace(prev)
 
     def run():
-        result.current = None
         set_result_state(ResultState.STARTING)
 
         def runner():
@@ -136,8 +153,9 @@ def use_thread(
                     # in case a new thread was started that also was waiting for the previous
                     # thread to st stop, we can finish this
                     return
-            error.current = None
-            result.current = None
+            # we previously set current to None, but if we do not do that, we can still render the old value
+            # while we can still show a loading indicator using the .state
+            # result.current = None
             set_result_state(ResultState.RUNNING)
 
             sig = inspect.signature(callback)
@@ -158,6 +176,7 @@ def use_thread(
                             try:
                                 with cancel_guard():
                                     result.current = next(value)
+                                    error.current = None
                             except StopIteration:
                                 break
                             # assigning to the ref doesn't trigger a rerender, so do it manually
@@ -166,14 +185,19 @@ def use_thread(
                             set_result_state(ResultState.FINISHED)
                     else:
                         result.current = value
+                        error.current = None
                         if threading.current_thread() == running_thread.current:
                             set_result_state(ResultState.FINISHED)
                 except Exception as e:
                     error.current = e
                     if threading.current_thread() == running_thread.current:
-                        set_result_state(ResultState.ERROR)
                         logger.exception(e)
+                        set_result_state(ResultState.ERROR)
                     return
+                except CancelledError:
+                    pass
+                    # this means this thread is cancelled not be request, but because
+                    # a new thread is running, we can ignore this
             finally:
                 if threading.current_thread() == running_thread.current:
                     running_thread.current = None
@@ -190,7 +214,7 @@ def use_thread(
 
         return cleanup
 
-    react.use_side_effect(run, dependencies + [counter])
+    solara.use_side_effect(run, dependencies + [counter])
     return Result[T](value=result.current, error=error.current, state=result_state, cancel=cancel.set, _retry=retry)
 
 
@@ -200,7 +224,7 @@ def use_download(
     if not isinstance(f, Result):
         f = Result(value=f)
     assert isinstance(f, Result)
-    content_length, set_content_length = react.use_state(expected_size, key="content_length")
+    content_length, set_content_length = solara.use_state(expected_size, key="content_length")
     downloaded_length = 0
     file_object = hasattr(f.value, "tell")
     if not file_object:
@@ -209,7 +233,7 @@ def use_download(
             file_size = os.path.getsize(file_path)
             if file_size == expected_size:
                 downloaded_length = file_size
-    downloaded_length, set_downloaded_length = react.use_state(downloaded_length, key="downloaded_length")
+    downloaded_length, set_downloaded_length = solara.use_state(downloaded_length, key="downloaded_length")
 
     def download(cancel: threading.Event):
         assert isinstance(f, Result)
@@ -219,7 +243,7 @@ def use_download(
 
         context: Any = None
         if file_object:
-            context = contextlib.nullcontext()
+            context = nullcontext()
             output_file = cast(IO, f.value)
         else:
             # f = cast(Result[Union[str, os.PathLike]], f)
@@ -257,9 +281,8 @@ def use_download(
 
 def use_fetch(url, chunk_size=chunk_size_default):
     # re-use the same file like object
-    f = react.use_memo(lambda *ignore: io.BytesIO(), args=[url])
+    f = solara.use_memo(io.BytesIO, [url])
     result = use_download(f, url, return_content=True, chunk_size=chunk_size)
-    print(result.progress)
     return dataclasses.replace(result, value=f.getvalue() if result.progress == 1 else None)
 
 
@@ -299,7 +322,6 @@ def use_json(path):
 def use_file_content(path, watch=False) -> FileContentResult[bytes]:
     counter, retry = use_retry()
 
-    @react.use_memo
     def read_file(*ignore):
         try:
             with open(path, "rb") as f:
@@ -307,18 +329,15 @@ def use_file_content(path, watch=False) -> FileContentResult[bytes]:
         except Exception as e:
             return e
 
+    result = None
     try:
         mtime = os.path.getmtime(path)
-    except Exception as e:
-        result = FileContentResult[bytes](error=e, _retry=retry)
-        # result.retry = retry
-        return result
+    except Exception:
+        mtime = None
 
-    content = read_file(
-        path,
-        mtime,
-        counter,
-    )
+    content = solara.use_memo(read_file, dependencies=[path, mtime, counter])
+    if result is not None:
+        return result
     if isinstance(content, Exception):
         return FileContentResult[bytes](error=content, _retry=retry)
     else:
@@ -330,7 +349,7 @@ def use_force_update() -> Callable[[], None]:
 
     This is used when external state has change, and we need to re-render out component.
     """
-    _counter, set_counter = react.use_state(0, "force update counter")
+    _counter, set_counter = solara.use_state(0, "force update counter")
 
     def updater():
         set_counter(lambda count: count + 1)
@@ -341,11 +360,10 @@ def use_force_update() -> Callable[[], None]:
 def use_uuid4(dependencies=[]):
     """Generate a unique string using the uuid4 algorithm. Will only change when the dependencies change."""
 
-    @react.use_memo
     def make_uuid(*_ignore):
         return str(uuid.uuid4())
 
-    return make_uuid(dependencies)
+    return solara.use_memo(make_uuid, dependencies)
 
 
 def use_unique_key(key: str = None, prefix: str = "", dependencies=[]):
@@ -360,15 +378,26 @@ def use_state_or_update(
     """This is useful for situations where a prop can change from a parent
     component, which should be respected, and otherwise the internal
     state should be kept.
-
-    See sol.Details for example usage.
     """
-    value, set_value = react.use_state(initial_or_updated, key=key, eq=eq)
+    value, set_value = solara.use_state(initial_or_updated, key=key, eq=eq)
 
-    def possibly_update(initial_or_updated):
+    def possibly_update():
+        nonlocal value
         # only gets called when initial_or_updated changes
-        # if value != initial_or_updated:
         set_value(initial_or_updated)
+        # this make sure the return value gets updated directly
+        value = initial_or_updated
 
-    react.use_memo(possibly_update)(initial_or_updated)
+    solara.use_memo(possibly_update, [initial_or_updated])
     return value, set_value
+
+
+def use_previous(value: T, condition=True) -> T:
+    ref = solara.use_ref(value)
+
+    def assign():
+        if condition:
+            ref.current = value
+
+    solara.use_effect(assign, [value])
+    return ref.current
