@@ -4,12 +4,13 @@ import sys
 import threading
 import traceback
 from typing import MutableMapping
+from unittest import mock
 
 import ipykernel.kernelbase
 import IPython.display
 import ipywidgets
 
-from . import app, settings
+from . import app, reload, settings
 
 logger = logging.getLogger("solara.server.app")
 
@@ -17,14 +18,29 @@ logger = logging.getLogger("solara.server.app")
 class FakeIPython:
     def __init__(self, context: app.AppContext):
         self.context = context
+        self.kernel = context.kernel
+        self.display_pub = mock.MagicMock()
+        # needed for the pyplot interface of matplotlib
+        # (although we don't really support it)
+        self.events = mock.MagicMock()
+
+    def enable_gui(self, gui):
+        logger.error("ignoring call to enable_gui(%s)", gui)
+
+    def register_post_execute(self, callback):
+        # mpl requires this
+        pass
+
+    def set_parent(self, *args):
+        pass
 
     def showtraceback(self):
-
         if settings.main.use_pdb:
             logger.exception("Exception, will be handled by debugger")
             pdb.post_mortem()
         etype, value, tb = sys.exc_info()
         traceback_string = "".join(traceback.format_exception(etype, value, tb))
+        logger.error("Uncaught exception: %s", traceback_string)
         msg = {
             "type": "exception",
             "traceback": traceback_string,
@@ -52,7 +68,7 @@ def kernel_initialized_dispatch(cls):
 
 
 def display_solara(*objs, include=None, exclude=None, metadata=None, transient=None, display_id=None, raw=False, clear=False, **kwargs):
-    print(*objs)
+    print(*objs)  # noqa
 
 
 # from IPython.core.interactiveshell import InteractiveShell
@@ -139,6 +155,28 @@ class context_dict_templates(context_dict):
         return context.templates
 
 
+class context_dict_user(context_dict):
+    def __init__(self, name):
+        self.name = name
+
+    def _get_context_dict(self) -> dict:
+        context = app.get_current_context()
+        if self.name not in context.user_dicts:
+            context.user_dicts[self.name] = {}
+        return context.user_dicts[self.name]
+
+
+def auto_watch_get_template(get_template):
+    """Wraps get_template and adds a file listener for automatic .vue file reloading"""
+
+    def wrapper(abs_path):
+        template = get_template(abs_path)
+        reload.reloader.watcher.add_file(abs_path)
+        return template
+
+    return wrapper
+
+
 Thread__init__ = threading.Thread.__init__
 Thread__run = threading.Thread.run
 
@@ -150,11 +188,11 @@ def WidgetContextAwareThread__init__(self, *args, **kwargs):
         self.current_context = app.get_current_context()
     except RuntimeError:
         logger.debug(f"No context for thread {self}")
-    if self.current_context:
-        app.set_context_for_thread(self.current_context, self)
 
 
 def Thread_debug_run(self):
+    if self.current_context:
+        app.set_context_for_thread(self.current_context, self)
     try:
         Thread__run(self)
     except Exception:
@@ -172,9 +210,29 @@ def patch():
     # because the import in ipvue overrides it
     template_mod = sys.modules["ipyvue.Template"]
     template_mod.template_registry = context_dict_templates()  # type: ignore
+    template_mod.get_template = auto_watch_get_template(template_mod.get_template)  # type: ignore
+
+    # this module also imports get_template
+    template_mod_vue = sys.modules["ipyvue.VueTemplateWidget"]
+    template_mod_vue.get_template = template_mod.get_template  # type: ignore
+
+    component_mod_vue = sys.modules["ipyvue.VueComponentRegistry"]
+    component_mod_vue.vue_component_registry = context_dict_user("vue_component_registry")  # type: ignore
+    component_mod_vue.vue_component_files = context_dict_user("vue_component_files")  # type: ignore
+
     ipywidgets.widget.Widget.widgets = context_dict_widgets()  # type: ignore
     threading.Thread.__init__ = WidgetContextAwareThread__init__  # type: ignore
     threading.Thread.run = Thread_debug_run  # type: ignore
-    ipykernel.kernelbase.Kernel.instance = classmethod(kernel_instance_dispatch)
-    ipykernel.kernelbase.Kernel.initialized = classmethod(kernel_initialized_dispatch)
+    # on CI we get a mypy error:
+    # solara/server/patch.py:210: error: Cannot assign to a method
+    #  solara/server/patch.py:210: error: Incompatible types in assignment (expression has type "classmethod[Any]",\
+    #                                     variable has type "Callable[[VarArg(Any), KwArg(Any)], Any]")
+    # not sure why we cannot reproduce that locally
+    ipykernel.kernelbase.Kernel.instance = classmethod(kernel_instance_dispatch)  # type: ignore
+    # on CI we get a mypy error:
+    # solara/server/patch.py:211: error: Cannot assign to a method
+    # solara/server/patch.py:211: error: Incompatible types in assignment (expression has type "classmethod[Any]", variable has type "Callable[[], Any]")
+    # not sure why we cannot reproduce that locally
+    ipykernel.kernelbase.Kernel.initialized = classmethod(kernel_initialized_dispatch)  # type: ignore
     ipywidgets.widgets.widget.get_ipython = get_ipython
+    IPython.get_ipython = get_ipython
